require "time"
require "queue/word_analyser"

module Enumerable
  def drop_until
    condition_met = false
    Enumerator.new do |yielder|
      self.each do |element|
        condition_met ||= yield(element)
        yielder << element if condition_met
      end
    end
  end
end

module AssemblyPipe; end

module AssemblyPipe::VelvetOptimiser

  class Run
    def initialize
      
    end
  end

  class LogFile
    def initialize(filename)
      @lines = File.readlines(filename).map{|line| line.strip}
    end
    
    def self.open(filename)
      self.new(filename)
    end
    
    def date
      Time.parse(@lines.first)
    end
    
    def velveth_runs
      Enumerator.new do |yielder|
        run = {}
        @lines.drop_until{|line| line.match(/Beginning velveth runs/)}.
          take_while{|line| !line.match(/Beginning vanilla velvetg runs/)}.
          each do |line|
          key, value = line.split(":",2)
          case key
          when /Velveth version/,
            /Readfile/,
            /Velveth parameter string/,
            /Assembly directory/
            run[key] = value.strip
          when /Velveth timestamp/
            run[key] = Time.parse(value)
          when /Assembly id/,
            /Velvet hash value/,
            /Roadmap file size/
            run[key] = value.to_i
          when /\*{10,}/
            yielder << run unless run.keys.empty?
            run = {}
          end
        end
      end
    end

    def vanilla_velvetg_runs
      Enumerator.new do |yielder|
        run = {}
        @lines.drop_until{|line| line.match(/Beginning vanilla velvetg runs/)}.
          take_while{|line| !line.match(/Best assembly by assembly score/)}.
          each do |line|
          key,value = line.split(":",2)
          case key
          when /Assembly id/,
            /Assembly score/,
            /Velvet hash value/,
            /Roadmap file size/,
            /Total number of contigs/,
            /n50/,
            /length of longest contig/,
            /Total bases in contigs/,
            /Number of contigs/,
            /Total bases in contigs/            
            run[key] = value.to_i
          when /timestamp/
            run[key] = Time.parse(value)
          when /version/,
            /Readfile/,
            /parameter string/,
            /Assembly directory/
            run[key] = value.strip
          when /\*{10,}/
            yielder << run unless run.keys.empty?
            run = {}
          end
        end
      end
    end

    def final_assembly
      run = {}
      @lines.drop_until{|line| line.match(/Final optimised assembly details/)}.
        take_while{|line| !line.match(/Assembly output files are in the following directory/)}.
        each do |line|
        key, value = line.split(":",2)
        case key
        when /Assembly id/,
          /Assembly score/,
          /Roadmap file size/,
          /Total number of contigs/,
          /n50/,
          /length of longest contig/,
          /Total bases in contigs/,
          /Number of contigs/,
          /Total bases in contigs/
          run[key] = value.to_i
        when /timestamp/
          run[key] = Time.parse(value)
        when /version/,
          /Readfile/,
          /parameter string/,
          /Velvet hash value/,
          /Paired Library insert stats/
          run[key] = value.strip
        end
      end
      run
    end

    def summary_table
      vvgs = vanilla_velvetg_runs.map{|run| run["Run class"] = "vanilla"; run}
      final = final_assembly
      final["Run class"] = "optimised"

      all_keys = vvgs.map{|run| run.keys}.flatten + final_assembly.keys
      all_keys.uniq!
      output = ''
      output << all_keys.join("\t") + "\n"
      vvgs.each do |run|
        output << all_keys.map{|key| run[key] }.join("\t") + "\n"
      end
      output << all_keys.map{|key| final[key] }.join("\t") + "\n"
    end
  end
end
